=== modified file 'NEWS'
--- NEWS	2008-06-05 05:53:31 +0000
+++ NEWS	2008-06-05 16:27:16 +0000
@@ -23,6 +23,11 @@
 
     * Added the 'alias' command to set/unset and display aliases. (Tim Penhey)
 
+    * ``added``, ``modified``, and ``unknowns`` behaviour made consistent (all three
+      now quote paths where required). Added ``--null`` option to ``added`` and 
+      ``modified`` (for null-separated unknowns, use ``ls --unknown --null``)
+      (Adrian Wilkins)
+
   BUGFIXES:
 
     * ``bzr status`` was breaking if you merged the same revision twice.
@@ -928,6 +933,8 @@
       (Alexander Belchenko)
 
   TESTING:
+    
+    * Added missing blackbox tests for ``modified`` (Adrian Wilkins)
 
     * The branch interface tests were invalid for branches using rich-root
       repositories because the empty string is not a valid file-id.

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2008-05-30 10:57:24 +0000
+++ bzrlib/builtins.py	2008-06-05 16:27:16 +0000
@@ -1585,13 +1585,21 @@
 
     hidden = True
     _see_also = ['status', 'ls']
+    takes_options = [
+            Option('null',
+                   help='Write an ascii NUL (\\0) separator '
+                   'between files rather than a newline.')
+            ]
 
     @display_command
-    def run(self):
+    def run(self, null=False):
         tree = WorkingTree.open_containing(u'.')[0]
         td = tree.changes_from(tree.basis_tree())
         for path, id, kind, text_modified, meta_modified in td.modified:
-            self.outf.write(path + '\n')
+            if null:
+                self.outf.write(path + '\0')
+            else:
+                self.outf.write(osutils.quotefn(path) + '\n')
 
 
 class cmd_added(Command):
@@ -1600,9 +1608,14 @@
 
     hidden = True
     _see_also = ['status', 'ls']
+    takes_options = [
+            Option('null',
+                   help='Write an ascii NUL (\\0) separator '
+                   'between files rather than a newline.')
+            ]
 
     @display_command
-    def run(self):
+    def run(self, null=False):
         wt = WorkingTree.open_containing(u'.')[0]
         wt.lock_read()
         try:
@@ -1619,7 +1632,10 @@
                     path = inv.id2path(file_id)
                     if not os.access(osutils.abspath(path), os.F_OK):
                         continue
-                    self.outf.write(path + '\n')
+                    if null:
+                        self.outf.write(path + '\0')
+                    else:
+                        self.outf.write(osutils.quotefn(path) + '\n')
             finally:
                 basis.unlock()
         finally:

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2008-05-30 10:57:24 +0000
+++ bzrlib/tests/blackbox/__init__.py	2008-06-05 16:27:16 +0000
@@ -81,6 +81,7 @@
                      'bzrlib.tests.blackbox.test_merge',
                      'bzrlib.tests.blackbox.test_merge_directive',
                      'bzrlib.tests.blackbox.test_missing',
+                     'bzrlib.tests.blackbox.test_modified',
                      'bzrlib.tests.blackbox.test_mv',
                      'bzrlib.tests.blackbox.test_nick',
                      'bzrlib.tests.blackbox.test_outside_wt',

=== modified file 'bzrlib/tests/blackbox/test_added.py'
--- bzrlib/tests/blackbox/test_added.py	2007-08-02 17:55:39 +0000
+++ bzrlib/tests/blackbox/test_added.py	2008-06-05 16:24:53 +0000
@@ -23,13 +23,30 @@
 from bzrlib.branch import Branch
 from bzrlib.tests.blackbox import ExternalBase
 
+
 class TestAdded(ExternalBase):
 
     def test_added(self):
         """Test that 'added' command reports added files"""
-
-        def check_added(expected):
-            out, err = self.run_bzr('added')
+        self._test_added('a', 'a\n')
+
+    def test_added_with_spaces(self):
+        """Test that 'added' command reports added files with spaces in their names quoted"""
+        self._test_added('a filename with spaces', '"a filename with spaces"\n')
+    
+    def test_added_null_separator(self):
+        """Test that added uses its null operator properly"""
+        self._test_added('a', 'a\0', null=True)
+
+    def _test_added(self, name, output, null=False):
+
+        def check_added(expected, null=False):
+            command = 'added'
+
+            if null:
+                command += ' --null'
+
+            out, err = self.run_bzr(command)
             self.assertEquals(out, expected)
             self.assertEquals(err, '')
 
@@ -38,15 +55,15 @@
         check_added('')
 
         # with unknown file, still nothing added
-        self.build_tree_contents([('a', 'contents of a\n')])
+        self.build_tree_contents([(name, 'contents of %s\n' % (name))])
         check_added('')
-
+        
         # after add, shows up in list
         # bug report 20060119 by Nathan McCallum -- 'bzr added' causes
         # NameError
-        tree.add('a')
-        check_added('a\n')
+        tree.add(name)
+        check_added(output, null) 
 
         # after commit, now no longer listed
-        tree.commit(message='add a')
+        tree.commit(message='add "%s"' % (name))
         check_added('')

=== added file 'bzrlib/tests/blackbox/test_modified.py'
--- bzrlib/tests/blackbox/test_modified.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_modified.py	2008-06-05 17:38:02 +0000
@@ -0,0 +1,72 @@
+# Copyright (C) 2006 Canonical Ltd
+# -*- coding: utf-8 -*-
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for 'bzr modified', which shows modified files."""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests.blackbox import ExternalBase
+
+class TestModified(ExternalBase):
+
+    def test_modified(self):
+        """Test that 'modified' command reports modified files"""
+        self._test_modified('a', 'a')
+
+    def test_modified_with_spaces(self):
+        """Test that 'modified' command reports modified files with spaces in their names quoted"""
+        self._test_modified('a filename with spaces', '"a filename with spaces"')
+
+    def _test_modified(self, name, output):
+
+        def check_modified(expected, null=False):
+            command = 'modified'
+            if null:
+                command += ' --null'
+            out, err = self.run_bzr(command)
+            self.assertEquals(out, expected)
+            self.assertEquals(err, '')
+
+        # in empty directory, nothing modified
+        tree = self.make_branch_and_tree('.')
+        check_modified('')
+
+        # with unknown file, still nothing modified
+        self.build_tree_contents([(name, 'contents of %s\n' % (name))])
+        check_modified('')
+        
+        # after add, not modified
+        tree.add(name)
+        check_modified('')
+
+        # after commit, not modified
+        tree.commit(message='add %s' % output)
+        check_modified('')
+
+        # modify the file
+        self.build_tree_contents([(name, 'changed\n')]) 
+        check_modified(output + '\n')
+        
+        # check null seps - use the unquoted raw name here
+        check_modified(name + '\0', null=True)
+
+        # now commit the file and it's no longer modified
+        tree.commit(message='modified %s' %(name))
+        check_modified('')
+

=== modified file 'bzrlib/tests/blackbox/test_non_ascii.py'
--- bzrlib/tests/blackbox/test_non_ascii.py	2007-12-18 15:23:12 +0000
+++ bzrlib/tests/blackbox/test_non_ascii.py	2008-05-21 11:40:35 +0000
@@ -362,7 +362,7 @@
         self.build_tree_contents([(fname, 'modified\n')])
 
         txt = self.run_bzr_decode('modified')
-        self.assertEqual(fname+'\n', txt)
+        self.assertEqual('"'+fname+'"'+'\n', txt)
 
         self.run_bzr_decode('modified', encoding='ascii', fail=True)
 
@@ -372,7 +372,7 @@
         self.wt.add(fname)
 
         txt = self.run_bzr_decode('added')
-        self.assertEqual(fname+'\n', txt)
+        self.assertEqual('"'+fname+'"'+'\n', txt)
 
         self.run_bzr_decode('added', encoding='ascii', fail=True)
 
@@ -439,6 +439,7 @@
 
         # TODO: jam 20060112 bzr unknowns is the only one which 
         #       quotes paths do we really want it to?
+        #       awilkins 20080521 added and modified do it now as well
         txt = self.run_bzr_decode('unknowns')
         self.assertEqual(u'"%s"\n' % (fname,), txt)
 

=== modified file 'bzrlib/tests/blackbox/test_unknowns.py'
--- bzrlib/tests/blackbox/test_unknowns.py	2007-09-05 01:54:07 +0000
+++ bzrlib/tests/blackbox/test_unknowns.py	2008-06-05 16:24:53 +0000
@@ -20,6 +20,7 @@
 
 from bzrlib.tests.blackbox import ExternalBase
 
+
 class TestUnknowns(ExternalBase):
 
     def test_unknowns(self):
@@ -33,12 +34,12 @@
         self.build_tree_contents([('a', 'contents of a\n')])
         self.assertEquals(self.run_bzr('unknowns')[0], 'a\n')
 
-        # multiple unknown files
-        self.build_tree(['b', 'c'])
-        self.assertEquals(self.run_bzr('unknowns')[0], 'a\nb\nc\n')
+        # multiple unknown files, including one with a space in its name
+        self.build_tree(['b', 'c', 'd e'])
+        self.assertEquals(self.run_bzr('unknowns')[0], 'a\nb\nc\n"d e"\n')
 
         # after add, file no longer shown
-        tree.add('a')
+        tree.add(['a', 'd e'])
         self.assertEquals(self.run_bzr('unknowns')[0], 'b\nc\n')
 
         # after all added, none shown

